@@ -1,6 +1,6 @@
 package Proc::tored::Manager;
 # ABSTRACT: OO interface to creating a proctored service
-$Proc::tored::Manager::VERSION = '0.07';
+$Proc::tored::Manager::VERSION = '0.08';
 
 use strict;
 use warnings;
@@ -11,41 +11,26 @@ use Guard qw(guard);
 use Path::Tiny qw(path);
 use Time::HiRes qw(sleep);
 use Try::Tiny;
-use Type::Utils qw(declare as where);
-use Types::Standard qw(Str Bool Num is_CodeRef);
-
-my $NonEmptyStr = declare, as Str, where { $_ =~ /\S/ };
-my $Directory = declare, as $NonEmptyStr, where { -d $_ && -w $_ };
-
-with 'Proc::tored::Role::Running';
-
+use Types::Standard -all;
+use Proc::tored::Types -types;
+use Proc::tored::Flag;
 
 
 has dir => (
   is  => 'ro',
-  isa => $Directory,
+  isa => Dir,
   required => 1,
 );
 
 has name => (
   is  => 'ro',
-  isa => $NonEmptyStr,
+  isa => NonEmptyStr,
   required => 1,
 );
 
-has '+term_file' => (
-  is => 'lazy',
-);
-
-sub _build_term_file {
-  my $self = shift;
-  my $file = path($self->dir)->child($self->name . '.term');
-  return "$file";
-}
-
 has pid_file => (
   is  => 'lazy',
-  isa => $NonEmptyStr,
+  isa => NonEmptyStr,
 );
 
 sub _build_pid_file {
@@ -56,7 +41,7 @@ sub _build_pid_file {
 
 has lock_file => (
   is  => 'lazy',
-  isa => $NonEmptyStr,
+  isa => NonEmptyStr,
 );
 
 sub _build_lock_file {
@@ -65,15 +50,66 @@ sub _build_lock_file {
   return "$file";
 }
 
+has stop_flag => (
+  is  => 'lazy',
+  isa => InstanceOf['Proc::tored::Flag'],
+  handles => {
+    stop => 'set',
+    start => 'unset',
+    is_stopped => 'is_set',
+  },
+);
+
+sub _build_stop_flag {
+  my $self = shift;
+  my $file = path($self->dir)->child($self->name . '.stopped');
+  Proc::tored::Flag->new(touch_file_path => "$file");
+}
+
+has pause_flag => (
+  is  => 'lazy',
+  isa => InstanceOf['Proc::tored::Flag'],
+  handles => {
+    pause => 'set',
+    resume => 'unset',
+    is_paused => 'is_set',
+  },
+);
+
+sub _build_pause_flag {
+  my $self = shift;
+  my $file = path($self->dir)->child($self->name . '.paused');
+  Proc::tored::Flag->new(touch_file_path => "$file");
+}
+
+
+sub clear_flags {
+  my $self = shift;
+  $self->start;
+  $self->resume;
+}
+
+
+sub is_running {
+  my $self = shift;
+  return $self->running_pid == $$;
+}
 
 
 sub service {
   my ($self, $code) = @_;
+  return 0 if $self->is_stopped;
+  return 0 if $self->running_pid;
   die 'expected a CODE ref' unless is_CodeRef($code);
 
   if (my $guard = $self->run_lock) {
-    while ($self->is_running && $code->()) {
-      ;
+    until ($self->is_stopped) {
+      if ($self->is_paused) {
+        sleep 0.2;
+        next;
+      }
+
+      last unless $code->();
     }
 
     return 1;
@@ -102,44 +138,50 @@ sub running_pid {
 }
 
 
-sub stop_running_process {
+sub stop_wait {
   my ($self, $timeout, $sleep) = @_;
   $sleep ||= 0.2;
 
-  my $pid = $self->running_pid || return 0;
-  return $self->stop if $pid == $$;
+  $self->stop;
+  return if $self->is_running;
 
-  if ($self->signal) {
-    if ($timeout) {
-      while (kill(0, $pid) && $timeout > 0) {
-        sleep $sleep;
-        $timeout -= $sleep;
-      }
-    }
+  my $pid = $self->running_pid || return 0;
+
+  while (kill(0, $pid) && $timeout > 0) {
+    sleep $sleep;
+    $timeout -= $sleep;
   }
 
   !kill(0, $pid);
 }
 
+#-------------------------------------------------------------------------------
+# Attempts to atomically acquire the run lock. Once held, the pid file is created
+# if needed, the current process' pid is written to it.
+#
+# If the lock is acquired, a L<Guard> object is returned that will release the
+# lock once out of scope. Returns undef otherwise.
+#
+# This method is I<not> used to determine if there is an existing process
+# running. It is I<only> used to safely manage the pid file while running.
+# service() should instead be used for coordinated launch of a service.
+#-------------------------------------------------------------------------------
 
 sub run_lock {
   my $self = shift;
-  return if $self->running_pid;
-  my $pid = $$;
+  return if $self->is_running;
+
   my $locked = $self->_lock;
 
   if ($locked) {
     # Write pid to the pidfile
     my $file = path($self->pid_file);
-    $file->spew("$pid\n");
-    $self->start;
+    $file->spew("$$\n");
 
     # Create guard object that releases the pidfile once out of scope
     return guard {
-      if ($$ == $pid) {
-        $self->stop;
-        $file->append({truncate => 1});
-      }
+      $file->append({truncate => 1})
+        if $self->is_running;
     };
   }
 
@@ -152,7 +194,6 @@ sub run_lock {
 #-------------------------------------------------------------------------------
 sub _lock {
   my $self = shift;
-  my $pid = $$;
 
   # Existing .lock file means another process came in ahead
   my $lock = path($self->lock_file);
@@ -171,7 +212,7 @@ sub _lock {
     };
 
   return unless $locked;
-  return guard { $self->_unlock if $$ == $pid };
+  return guard { $self->_unlock if $self->is_running };
 }
 
 #-------------------------------------------------------------------------------
@@ -197,7 +238,7 @@ Proc::tored::Manager - OO interface to creating a proctored service
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -209,7 +250,7 @@ version 0.07
 
   # Signal another process running this service to quit gracefully, throwing an
   # error if it does not self-terminate after 15 seconds.
-  if (my $pid = $proctor->stop_running_process(15)) {
+  if (my $pid = $proctor->stop_wait(15)) {
     die "process $pid is being stubborn!";
   }
 
@@ -237,12 +278,6 @@ pid file.
 A valid directory path where the pid file is to be created or an existing pid
 file is to be found.
 
-=item term_file
-
-A touch file is used to signal a process to self-terminate. The path to this
-file is automatically constructed from L</name> in L</dir> unless manually
-specified.
-
 =item lock_file
 
 Before writing the pid file, a lock is secured through the atomic creation of a
@@ -255,18 +290,48 @@ L</name> in L</dir>.
 
 =back
 
+=head1 METHODS
+
+=head2 stop
+
+Sets the "stopped" flag for the service.
+
+=head2 start
+
+Clears the "stopped" flag for the service.
+
+=head2 is_stopped
+
+Returns true if the "stopped" flag has been set.
+
+=head2 pause
+
+Sets the "paused" flag for the service.
+
+=head2 resume
+
+Clears the "paused" flag for the service.
+
+=head2 is_paused
+
+Returns true if the "paused" flag has been set.
+
+=head2 clear_flags
+
+Clears both the "stopped" and "paused" flags.
+
 =head2 is_running
 
-See L<Proc::tored::Role::Running/is_running>.
+Returns true if the current process is the active, running process.
 
 =head2 service
 
-Accepts a code ref which will be called repeatedly until it or L</is_running>
-return false.
+Accepts a code ref which will be called repeatedly until it returns false or
+the "stopped" flag is set. If the "paused" flag is set, will continue to rune
+but will not execute the code block until the "paused" flag has been cleared.
 
 Example using a pool of forked workers, an imaginary task queue, and a
-secondary condition that decides whether to stop running (aside from the
-built-in signal handlers):
+secondary condition that decides whether to stop running.
 
   $proctor->service(sub {
     # Wait for an available worker, but with a timeout
@@ -296,33 +361,12 @@ Returns the pid of an already-running process or 0 if the pid file does not
 exist, is empty, or the process identified by the pid does not exist or is not
 visible.
 
-=head2 stop_running_process
+=head2 stop_wait
 
-Signals a running instance to self-terminate. Returns 0 immediately if the pid
-file does not exist or is empty. Otherwise, polls the running process until the
-OS reports that it is no longer able to receive signals (with `kill(0, $pid)`).
+Sets the "stopped" flag and blocks until the L<running_pid> exits or the
+C<$timeout> is reached.
 
-Optional parameter C<$timeout> may be specified in fractional seconds, causing
-C<stop_running_process> to block up to (around) C<$timeout> seconds waiting for
-the signaled process to exit. Optional parameter C<$sleep> specifies the
-interval between polls in fractional seconds.
-
-Returns the pid of the completed process otherwise.
-
-  $service->stop_running_process; # signal running process and return
-  $service->stop_running_process(10, 0.5); # signal, then poll every 0.5s for 10s
-
-=head2 run_lock
-
-Attempts to atomically acquire the run lock. Once held, the pid file is created
-(if needed) and the current process' pid is written to it, L</is_running> will
-return true and a signal handlers will be active. Existing handlers will be
-executed after the one assigned for the run lock.
-
-If the lock is acquired, a L<Guard> object is returned that will release the
-lock once out of scope. Returns undef otherwise.
-
-L</service> is preferred to this method for most uses.
+  $service->stop_wait(30); # stop and block for up to 30 seconds
 
 =head1 AUTHOR
 
